/* /////////////////////////////////////////////////////////////////////////////
 * File:        explicit_cast_test.cpp
 *
 * Purpose:     Implementation file for the explicit_cast_test project.
 *
 * Created:     31st July 2003
 * Updated:     30th May 2004
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright (C) 1999-2003, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2003
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software.
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage.
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#include <stdio.h>

#include <stlsoft.h>
#include <stlsoft_explicit_cast.h>

stlsoft_ns_using(ss_sint64_t)
stlsoft_ns_using(explicit_cast)

void fi(int i);
void fi64(ss_sint64_t i);
void fld(long double i);

struct Int
{
	Int(int i)
		: m_i(i)
	{}

	operator explicit_cast<int> () const
	{
		return m_i;
	}

	operator explicit_cast<int const &> () const
	{
		return m_i;
	}

private:
	int	m_i;
};

struct Int64
{
	Int64(ss_sint64_t i)
		: m_i(i)
	{}

	operator explicit_cast<ss_sint64_t> () const
	{
		return m_i;
	}

	operator explicit_cast<ss_sint64_t const &> () const
	{
		return m_i;
	}

private:
	ss_sint64_t	m_i;
};

struct LongDouble
{
	LongDouble(long double i)
		: m_i(i)
	{}

	operator explicit_cast<long double> () const
	{
		return m_i;
	}

	operator explicit_cast<long double const &> () const
	{
		return m_i;
	}

private:
	long double m_i;
};

void test_fi(int argc)
{
	{
		Int	i(argc);

		fi(explicit_cast<int>(i));
		fi(explicit_cast<int const &>(i));
	}

	{
		Int64	i(argc);

		fi64(explicit_cast<ss_sint64_t>(i));
		fi64(explicit_cast<ss_sint64_t const &>(i));
	}

	{
		LongDouble	i(argc);

		fld(explicit_cast<long double>(i));
		fld(explicit_cast<long double const &>(i));
	}
}


#if 0

class Value
{
public:
	Value()
		: m_id(_next_id())
	{
		printf("  Value::Value() -                  (%d)\n", m_id);
	}
	Value(Value const &v)
		: m_id(_next_id())
	{
		printf("  Value::Value(Value const &) -     (%d, %d)\n", m_id, v.m_id);
	}
	~Value()
	{
		printf("  Value::~Value() -                 (%d)\n", m_id);
	}

	Value &operator =(Value const &v)
	{
		printf("  Value::op =(Value const &) -      (%d, %d)\n", m_id, v.m_id);

		return *this;
	}

private:
	static int _next_id()
	{
		static int	s_i = 0;

		return ++s_i;
	}

public:
	int	m_id;
};

class ByValue
{
public:
	ByValue()
	{
	}

public:
	operator explicit_cast<Value> () const
	{
		printf("  ByValue::explicit_cast<Value>()   (%d)\n", m_value.m_id);

		return m_value;
	}

private:
	Value	m_value;
};

#if 0
class ByRef
{
public:
	operator explicit_cast<Value &> const ()
	{
		printf("  ByRef::explicit_cast<Value>()     (%d)\n", m_value.m_id);

		return m_value;
	}

private:
	Value	m_value;
};
#endif /* 0 */

class ByRefConst
{
public:
	operator explicit_cast<Value const &> () const
	{
		printf("  ByConstRef::explicit_cast<Value>()(%d)\n", m_value.m_id);

		return m_value;
	}

private:
	Value	m_value;
};

class ByPtr
{
public:
	operator explicit_cast<Value *> ()
	{
		printf("  ByPtr::explicit_cast<Value>()     (%d)\n", m_value.m_id);

		return &m_value;
	}

private:
	Value	m_value;
};

class ByPtrConst
{
public:
	operator explicit_cast<Value const *> () const
	{
		printf("  ByConstPtr::explicit_cast<Value>()(%d)\n", m_value.m_id);

		return &m_value;
	}

private:
	Value	m_value;
};

void f1(Value v);
void f2(Value &v);
void f3(Value const &v);
void f4(Value *pv);
void f5(Value const *pv);

void f1(Value v)
{
	printf(" f1() - (%d)\n", v.m_id);
}

void f2(Value &v)
{
	printf(" f2() - (%d)\n", v.m_id);
}

void f3(Value const &v)
{
	printf(" f3() - (%d)\n", v.m_id);
}

void f4(Value *pv)
{
	printf(" f3() - (%d)\n", pv->m_id);
}

void f5(Value const *pv)
{
	printf(" f3() - (%d)\n", pv->m_id);
}

//#include <windows.h>

int main(int argc, char **)
{
//	::Sleep(100000);

	puts("explicit_cast_test: " __STLSOFT_COMPILER_LABEL_STRING);

	test_fi(argc);

	printf("\nConstructing instances\n\n");

	ByValue		byvalue;
//	ByRef		byref;
	ByRefConst	byrefconst;
	ByPtr		byptr;
	ByPtrConst	byptrconst;

	printf("\nReturning into a value\n");

	Value		v0 = explicit_cast<Value>(byvalue);

	printf("\nby Value\n");

	printf("\n Calling f1\n");
	f1(explicit_cast<Value>(byvalue));
//	printf("\n Calling f2\n");
//	f2(explicit_cast<Value>(byvalue));
	printf("\n Calling f3\n");
	f3(explicit_cast<Value>(byvalue));
//	printf("\n Calling f4\n");
//	f4(&explicit_cast<Value>(byvalue));
	printf("\n Calling f5\n");
//	f5(&static_cast<Value>(explicit_cast<Value>(byvalue)));

	printf("\nby Value &\n");

#if 0
	printf("\n Calling f1\n");
	f1(explicit_cast<Value&>(byref));
//	printf("\n Calling f2\n");
//	f2(explicit_cast<Value&>(byref));
	printf("\n Calling f3\n");
	f3(explicit_cast<Value&>(byref));
//	printf("\n Calling f4\n");
//	f4(&explicit_cast<Value&>(byref));
	printf("\n Calling f5\n");
	f5(&static_cast<Value const&>(explicit_cast<Value&>(byref)));
#endif /* 0 */

	printf("\nby Value const &\n");

	printf("\n Calling f1\n");
	f1(explicit_cast<Value const &>(byrefconst));
//	printf("\n Calling f2\n");
//	f2(explicit_cast<Value const &>(byrefconst));
	printf("\n Calling f3\n");
	f3(explicit_cast<Value const &>(byrefconst));
//	printf("\n Calling f4\n");
//	f4(&explicit_cast<Value const &>(byrefconst));
	printf("\n Calling f5\n");
//	f5(&static_cast<Value const &>(explicit_cast<Value const &>(byrefconst)));

	printf("\nby Value *\n");

	printf("\n Calling f1\n");
	f1(*explicit_cast<Value*>(byptr));
//	printf("\n Calling f2\n");
//	f2(explicit_cast<Value*>(byptr));
	printf("\n Calling f3\n");
	f3(*explicit_cast<Value*>(byptr));
	printf("\n Calling f4\n");
	f4(explicit_cast<Value*>(byptr));
	printf("\n Calling f5\n");
	f5(explicit_cast<Value*>(byptr));

	printf("\nby Value const *\n");

	printf("\n Calling f1\n");
	f1(*explicit_cast<Value const *>(byptrconst));
//	printf("\n Calling f2\n");
//	f2(explicit_cast<Value const *>(byptrconst));
	printf("\n Calling f3\n");
	f3(*explicit_cast<Value const *>(byptrconst));
//	printf("\n Calling f4\n");
//	f4(explicit_cast<Value*>(byptrconst));
	printf("\n Calling f5\n");
	f5(explicit_cast<Value const *>(byptrconst));

#if 1
	printf("\n Getting reference\n");
	Value const &v	=	(explicit_cast<Value const &>)byrefconst;
	printf(" Calling f3() indirect)\n");

	f3(v);

	printf("\nConstructing v2 instance\n");
	Value	v2;

	printf("\n Calling f3() direct)\n");
	f3(v2);

	printf("\nDeclaring v3 as reference to v2\n");
	Value const &v3	=	v2;

	printf("\n Calling f3() direct)\n");
	f3(v2);

	STLSOFT_SUPPRESS_UNUSED(v3);
#endif /* 0 */

	printf("\nDestroying instances\n");

	return 0;
}

#else

#include <time.h>

typedef double	DATE;

#include <stlsoft_simple_string.h>

typedef	stlsoft_ns_qual(basic_simple_string)<char>	string_t;

#include <vector>

typedef stlsoft_ns_qual_std(vector)<int>  int_vector_t;


/* ////////////////////////////////////////////////////////////////////////// */

void f1(tm t);
void f2(DATE t);
void f3(string_t const &s);

/* ////////////////////////////////////////////////////////////////////////// */

struct tm_cast
{
//private:
	tm_cast(tm v)
		: m_v(v)
	{}

//  public:
	operator tm() const
	{
		return m_v;
	}

	tm m_v;
};

struct DATE_cast
{
	DATE_cast(DATE v)
		: m_v(v)
	{}

	operator DATE() const
	{
		return m_v;
	}

	DATE m_v;
};

class Time
{
public:
public:
	Time()
		: m_tm()
		, m_date(0.0)
		, m_str("member string")
	{
		printf("&m_tm: %p\n", &m_tm);
		printf("&m_date: %p\n", &m_date);
		printf("&m_str: %p\n", &m_str);

		time_t	t;

		time(&t);
		m_tm = *localtime(&t);
	}
public:
#if 0
	operator tm() const
	{
		return tm();
	}
	operator DATE() const
	{
		return 0.0;
	}
#else
	operator tm_cast() const
	{
		return m_tm;
	}
	operator DATE_cast() const
	{
		return m_date;
	}
	operator explicit_cast<DATE>() const
	{
		return m_date;
	}
	operator explicit_cast<DATE const &>() const
	{
		return m_date;
	}
#if 0
	operator explicit_cast<int_vector_t> () const
	{
		return int_vector_t();
	}
#endif /* 0 */
#endif /* 0 */

private:
	tm			m_tm;
	DATE		m_date;
	string_t	m_str;
};

void f1(tm t)
{
	printf("%04d %02d %02d\n", t.tm_year + 1900, t.tm_mon, t.tm_mday);
}

void f2(DATE /* t */)
{
}

void f3(string_t const &s)
{
	printf("&s: %p\n", &s);
}

void f4(string_t &s)
{
	printf("&s: %p\n", &s);
}

/* ////////////////////////////////////////////////////////////////////////// */

void dump_iv(int_vector_t const &iv)
{
	printf("[ ");
	for(size_t i = 0; i < iv.size(); ++i)
	{
		printf("%d ", iv[i]);
	}
	printf("]\n");
}

class ResourceManager
{
public:
	ResourceManager()
		: m_iv(3)
	{
		m_iv[0] = 1;
		m_iv[1] = 2;
		m_iv[2] = 3;
	}

public:
#if 0
  operator explicit_cast<int_vector_t> () const
  {
	  return m_iv;
  }
#endif /* 0 */

private:
	int_vector_t	m_iv;
};

void f(ResourceManager const &rm)
{
//  dump_iv(explicit_cast<int_vector_t>(rm)); // Expensive copy - ouch!
}

/* ////////////////////////////////////////////////////////////////////////// */

extern "C" void __stdcall Sleep(unsigned long);

int main(int argc, char ** /*argv*/)
{
	test_fi(argc);

	{
		ResourceManager	rm;

		f(rm);
	}


	{

//		::Sleep(100000);

		/* . */
		Time		t;
		Time const	&ct = t;

#if 0
		f1(t);
		f2(t);
#else
		f1(static_cast<tm_cast>(t));
		f2(static_cast<DATE_cast>(t));
		f2(explicit_cast<DATE>(t));
		f2(explicit_cast<DATE const &>(t));

		f1(static_cast<tm_cast>(ct));
		f2(static_cast<DATE_cast>(ct));
#endif /* 0 */
	}

    return 0;
}

/* ////////////////////////////////////////////////////////////////////////// */
#endif /* 0 */


void fi(int i)
{
	printf("fi(%d)\n", i);
}

void fi64(ss_sint64_t i)
{
	printf("fi64(%d)\n", (int)i);
}

void fld(long double i)
{
	printf("fld(%d)\n", (int)i);
}

/* ////////////////////////////////////////////////////////////////////////// */
