
// Updated: 9th September 2005

/*
 * test_stlsoft_null.cpp - test STLSoft's NULL_v real null pointer.
 *
 * Visual C++ 7.1:
 * prompt>cl -GX -D_STLSOFT_NULL_v_DEFINED -I D:/Libraries/stl-1.8.3/include
test_stlsoft_null.cpp
 * This compiler silently ignores our request for NULL as NULL_v (!).
 *
 * GCC 3.2.3:
 * prompt>g++.exe -c -I D:/Libraries/stl-1.8.3/include test_stlsoft_null.cpp
 * test_stlsoft_null.cpp: In function `int main()':
 * test_stlsoft_null.cpp:33: cannot convert `stlsoft::NULL_v' to `int' in
assignment
 * test_stlsoft_null.cpp:39: cannot convert `stlsoft::NULL_v' to `int
(A::*)()' in assignment
 */

//#define _STLSOFT_COMPILE_VERBOSE 1

#include <stdio.h>

#include <stlsoft/null.hpp>
#include <stlsoft_nulldef.h>

struct A
{
   int f();
};

int main()
{
   using stlsoft::NULL_v;

   int  i  = 0;                         // integer
   int *pi = &i;                        // pointer to integer
   int (A::*fp)() = &A::f;              // member function pointer

#if 0
   // Ok: fails
//   i = NULL_v();

   // Ok: succeeds:
   pi = NULL_v();

#ifdef _STLSOFT_NULL_v_DEFINED_PTR_TO_MEMBER_SUPPORT
   // Error: fails: test_stlsoft_null.cpp:41: cannot convert
//`stlsoft::NULL_v' to `int (A::*)()' in assignment
   fp = NULL_v();
#endif /* _STLSOFT_NULL_v_DEFINED_PTR_TO_MEMBER_SUPPORT */

   if(NULL_v() == 1)
   {
   }

   // Error: should fail (integer)
   if ( i == NULL_v() ) { }

   // Error: should fail (integer)
   if ( NULL_v() == i ) { }

   // Ok: succeeds (pointer):
   if ( pi == NULL_v() ) { }

   // Ok: succeeds (pointer):
   if ( NULL_v() == pi ) { }

#ifdef _STLSOFT_NULL_v_DEFINED_PTR_TO_MEMBER_SUPPORT
   // Ok: succeeds (member pointer):
   if ( fp == NULL_v() ) { }

   // Ok: succeeds (member pointer):
   if ( NULL_v() == fp ) { }
#endif /* _STLSOFT_NULL_v_DEFINED_PTR_TO_MEMBER_SUPPORT */

#else /* ? 0 */

   // Ok: fails
//   i = NULL;

   // Ok: succeeds:
   pi = NULL;

#ifdef _STLSOFT_NULL_v_DEFINED_PTR_TO_MEMBER_SUPPORT
   // Error: fails: test_stlsoft_null.cpp:41: cannot convert
//`stlsoft::NULL_v' to `int (A::*)()' in assignment
   fp = NULL;
#endif /* _STLSOFT_NULL_v_DEFINED_PTR_TO_MEMBER_SUPPORT */

#if 0
   if(NULL == 1)
   {
   }
#endif /* 0 */

#if 0
   // Error: should fail (integer)
   if ( i == NULL ) { }
#endif /* 0 */

#if 0
   // Error: should fail (integer)
   if ( NULL == i ) { }
#endif /* 0 */

#if 1
   // Ok: succeeds (pointer):
   if ( pi == NULL ) { }

   // Ok: succeeds (pointer):
   if ( NULL == pi ) { }
#endif /* 0 */

#if 0
#ifdef _STLSOFT_NULL_v_DEFINED_PTR_TO_MEMBER_SUPPORT
   // Ok: succeeds (member pointer):
   if ( fp == NULL ) { }

   // Ok: succeeds (member pointer):
   if ( NULL == fp ) { }
#endif /* _STLSOFT_NULL_v_DEFINED_PTR_TO_MEMBER_SUPPORT */
#endif /* 0 */


#endif /* 0 */

   return 0;
}
