/* /////////////////////////////////////////////////////////////////////////////
 * File:        pod_vector_test.cpp
 *
 * Purpose:     Implementation file for the pod_vector_test project.
 *
 * Created:     23rd December 2003
 * Updated:     8th July 2005
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright (C) 1999-2003, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2003
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software.
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage.
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission.
 *
 * ////////////////////////////////////////////////////////////////////////// */

#include <stdio.h>

#define	_STLSOFT_NO_NAMESPACE

#include <stlsoft.h>
#include <stlsoft/pod_vector.hpp>

/* ////////////////////////////////////////////////////////////////////////// */

#ifdef __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT
typedef	stlsoft_ns_qual(pod_vector)<int>												int_vector_t;
#else
typedef	stlsoft_ns_qual(pod_vector)<int, stlsoft_ns_qual(malloc_allocator)<int>, 64>	int_vector_t;
#endif /* __STLSOFT_CF_TEMPLATE_CLASS_DEFAULT_CLASS_ARGUMENT_SUPPORT */

# define for_each_		std::for_each

/* ////////////////////////////////////////////////////////////////////////// */

void print_int(int const &i)
{
	printf("%d ", i);
}

void dump(int_vector_t const &vi, char const *message)
{
	printf("%s\nSize: %lu\n", message, vi.size());
	printf("Contents (forward): ");
	for_each_(vi.begin(), vi.end(), print_int);
	printf("\n");
#if defined(__STLSOFT_CF_BIDIRECTIONAL_ITERATOR_SUPPORT)
	printf("Contents (backward): ");
	for_each_(vi.rbegin(), vi.rend(), print_int);
	printf("\n");
#endif /* __STLSOFT_CF_BIDIRECTIONAL_ITERATOR_SUPPORT */
	if(0 != vi.size())
	{
		printf("front: %d\n", vi.front());
		printf("\n");
		printf("back: %d\n", vi.back());
		printf("\n");
	}
	printf("\n");
}

/* ////////////////////////////////////////////////////////////////////////// */

extern "C" __declspec(dllimport) void __stdcall Sleep(unsigned long);

int main(int /* argc */, char ** /*argv*/)
{
#if 0
	::Sleep(100000);
#endif /* 0 */

	puts("pod_vector_test: " __STLSOFT_COMPILER_LABEL_STRING);

	/* . */
	int_vector_t	vi;
	int_vector_t	vi2(20);
	int_vector_t	vi3(20, 33);
	int_vector_t	vi4(vi3);

	dump(vi, "After construction");

	vi.resize(120);
	dump(vi, "After resize(120)");

	vi.resize(10);
	dump(vi, "After resize(10)");

	vi.resize(20, 9999);
	dump(vi, "After resize(20, 9999)");

	vi[2] = vi.at(1);
	dump(vi, "After vi[2] = vi.at(2);");

	vi[3] = 3;
	dump(vi, "After vi[3] = 3");

	vi.assign(2, 4);
	dump(vi, "After vi.assign(2, 4)");

	vi.insert(&vi[1], 3);
	dump(vi, "After vi.insert(&vi[1], 3)");

	vi.insert(&vi[2], 2, 19);
	dump(vi, "After vi.insert(&vi[2], 2, 19)");

	vi.insert(&vi[3], &vi3[0], &vi3[3]);
	dump(vi, "After vi.insert(&vi[3], &vi3[0], &vi3[3])");

	vi.erase(&vi[3]);
	dump(vi, "After vi.erase(&vi[3])");

	vi.erase(&vi[2], vi.end());
	dump(vi, "After vi.erase(&vi[2], vi.end())");

	vi.get_allocator();

	vi.swap(vi2);

    return 0;
}

/* ////////////////////////////////////////////////////////////////////////// */
