/* /////////////////////////////////////////////////////////////////////////////
 * File:        unused_return_value_monitor_test.cpp
 *
 * Purpose:     Implementation file for the unused_return_value_monitor_test project.
 *
 * Created:     9th November 2003
 * Updated:     22nd April 2004
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright (C) 1999-2003, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2003
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software.
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage.
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission.
 *
 * ////////////////////////////////////////////////////////////////////////// */

#include <stdio.h>

#include <stlsoft.h>
#include <stlsoft_unused_return_value_monitor.h>

stlsoft_ns_using(unused_return_value_monitor)

#include <string>

stlsoft_ns_using_std(string)

/* ////////////////////////////////////////////////////////////////////////// */

struct int_monitor
{
public:
	void operator ()(void const *instance, int value) const
	{
		printf("Unused return value %d from object instance %p\n", value, instance);
	}
};

struct string_monitor
{
public:
	void operator ()(void const *instance, string const &value) const
	{
		printf("Unused return value %s from object instance %p\n", value.c_str(), instance);
	}
};


class X
{
public:
	unused_return_value_monitor<int, int_monitor>	fn()
	{
		return 10;
	}
};

class Y1
{
public:
	unused_return_value_monitor<string, string_monitor> fn()
	{
		return "Hello Natty!";
	}
};

class Y2
{
public:
	Y2()
		: m_str("Hello Natty!")
	{}

public:
	unused_return_value_monitor<string, string_monitor, string const &> fn()
	{
		return m_str;
	}

private:
	string const	m_str;
};

class Z
{
public:
	Z()
		: m_val(0)
	{}
public:
	Z &operator ++()
	{
		++m_val;

		return *this;
	}

private:
	struct monitor
	{
	public:
		void operator ()(void const *instance, Z value) const
		{
			printf("Unused return value %p from Z instance %p\n", &value, instance);
		}
	};

public:
	unused_return_value_monitor<Z, monitor>	operator ++(int)
	{
		Z	r(*this);

		++m_val;

		return r;
	}

private:
	int	m_val;
};

/* ////////////////////////////////////////////////////////////////////////// */

int main(int /* argc */, char ** /*argv*/)
{
	puts("unused_return_value_monitor_test: " __STLSOFT_COMPILER_LABEL_STRING);

	// int
	X		x;
	int		i;

	i = x.fn();	// Used
	    x.fn();	// Unused

	// string
	Y1		y1;
	Y2		y2;
	string	s;

	s = y1.fn();	// Used
	    y1.fn();	// Unused
	s = y2.fn();	// Used
	    y2.fn();	// Unused

	// op ++
	Z		z;

	++z;

	Z		z1	=	z++; // Used
					z++; // Unused

    return 0;
}

/* ////////////////////////////////////////////////////////////////////////// */
