/* /////////////////////////////////////////////////////////////////////////////
 * File:        exceptions_test.cpp
 *
 * Purpose:     Implementation file for the exceptions_test project.
 *
 * Created:     5th April 2005
 * Updated:     5th April 2005
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright 1999-2005, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2005
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software. 
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage. 
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission. 
 *
 * ////////////////////////////////////////////////////////////////////////// */

/* STLSoft Header Files */
#include <stlsoft.h>

#include <unixstl.h>
#include <unixstl/exceptions.hpp>

/* Standard C Header Files */
#include <stdio.h>
#include <stdlib.h>

/* Standard C++ Header Files */
#include <exception>

#ifdef __WATCOMC__
namespace std
{
	using ::exception;
}
#endif /* __WATCOMC__ */

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

static void usage(int bExit, char const *reason, int iInvalidArg, int argc, char **argv);


/* ////////////////////////////////////////////////////////////////////////// */

static int main_(int argc, char *argv[])
{
    int                 iRet    =   0;
	int					bVerbose	=	true;
    int                 i;

#if defined(_DEBUG) || \
    defined(__SYNSOFT_DBS_DEBUG)
	puts("exceptions_test: " __STLSOFT_COMPILER_LABEL_STRING);
#endif /* debug */

	for(i = 1; i < argc; ++i)
	{
		const char	*const	arg	=	argv[i];

		if(arg[0] == '-')
		{
			if(arg[1] == '-')
			{
				/* -- arguments */
				usage(1, "Invalid argument(s) specified", i, argc, argv);
			}
			else
			{
				/* - arguments */
				switch(arg[1])
				{
					case	'?':
						usage(1, NULL, -1, argc, argv);
						break;
					case	's':
						bVerbose	=	false;
						break;
					case	'v':
						bVerbose	=	true;
						break;
					default:
						usage(1, "Invalid argument(s) specified", i, argc, argv);
						break;
				}
			}
		}
		else
		{
			/* other arguments */
			usage(1, "Invalid argument(s) specified", i, argc, argv);
		}
	}

	/* . */
	static const int	errors[] = 
	{
			0
		,	ENOMEM
		,	EACCES
		,	EBADF
		,	16777216
	};

	{ for(size_t i = 0; i < stlsoft_num_elements(errors); ++i)
	{
		try
		{
			printf("%12d => ", errors[i]);

//			throw ::unixstl::unix_exception(errors[i]);
//			throw ::unixstl::unix_exception("", errors[i]);
			throw ::unixstl::unix_exception("Any old message", errors[i]);
		}
		catch(std::exception &x)
		{
			printf("%s\n", x.what());
			printf("                %s\n", x.what());
		}
	}}


    return iRet;
}

//#include <windows.h>

int main(int argc, char *argv[])
{
#if 0
	::Sleep(100000);
#endif /* 0 */

	try
	{
		return main_(argc, argv);
	}
	catch(std::exception &x)
	{
		fprintf(stderr, "Unhandled error: %s\n", x.what());
	}
	catch(...)
	{
		fprintf(stderr, "Unhandled unknown error\n");
	}

	return EXIT_FAILURE;
}

/* ////////////////////////////////////////////////////////////////////////// */

static void usage(int bExit, char const *reason, int iInvalidArg, int argc, char **argv)
{

    fprintf(stderr, "\n");
    fprintf(stderr, "  exceptions_test\n\n");
    fprintf(stderr, "\n");

	if(NULL != reason)
	{
		fprintf(stderr, "  Error: %s\n", reason);
		fprintf(stderr, "\n");
	}

#if 0
    fprintf(stderr, "  USAGE: exceptions_test [{-w | -p<root-paths> | -h}] [-u] [-d] [{-v | -s}] <search-spec>\n");
    fprintf(stderr, "    where:\n\n");
    fprintf(stderr, "    -w             - searches from the current working directory. The default\n");
    fprintf(stderr, "    -p<root-paths> - searches from the given root path(s), separated by \';\',\n");
    fprintf(stderr, "                     eg.\n");
    fprintf(stderr, "                       -p\"c:\\windows;x:\\bin\"\n");
    fprintf(stderr, "    -r             - deletes readonly files\n");
    fprintf(stderr, "    -h             - searches from the roots of all drives on the system\n");
    fprintf(stderr, "    -d             - displays the path(s) searched\n");
    fprintf(stderr, "    -u             - do not act recursively\n");
    fprintf(stderr, "    -v             - verbose output. Prints time, attributes, size and path. (default)\n");
    fprintf(stderr, "    -s             - succinct output. Prints path only\n");
    fprintf(stderr, "    <search-spec>  - one or more file search specifications, separated by \';\',\n");
    fprintf(stderr, "                     eg.\n");
    fprintf(stderr, "                       \"*.exe\"\n");
    fprintf(stderr, "                       \"myfile.ext\"\n");
    fprintf(stderr, "                       \"*.exe;*.dll\"\n");
    fprintf(stderr, "                       \"*.xl?;report.*\"\n");
    fprintf(stderr, "\n");
	fprintf(stderr, "  Contact %s\n", _nameSynesisSoftware);
	fprintf(stderr, "    at \"%s\",\n", _wwwSynesisSoftware_SystemTools);
	fprintf(stderr, "    or, via email, at \"%s\"\n", _emailSynesisSoftware_SystemTools);
    fprintf(stderr, "\n");
#endif /* 0 */

    if(bExit)
    {
        exit(EXIT_FAILURE);
    }
}

/* ////////////////////////////////////////////////////////////////////////// */
