/* /////////////////////////////////////////////////////////////////////////////
 * File:        winstl_service_test.cpp
 *
 * Purpose:     Implementation file for the winstl_service_test project.
 *
 * Created:     5th November 2004
 * Updated:     6th November 2004
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright (C) 2004, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2004
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software. 
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage. 
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission. 
 *
 * ////////////////////////////////////////////////////////////////////////// */

/* STLSoft Header Files */
#include <stlsoft.h>

/* WinSTL Header Files */
#include <winstl.h>
#include <winstl_error_desc.h>

#include <winstl/services/services.hpp>
#include <winstl/services/handle.hpp>
#include <winstl/services/manager.hpp>
#include <winstl/services/service.hpp>
#include <winstl/services/service_exception.hpp>

/* Standard C Header Files */
#include <stdlib.h>
#include <windows.h>

/* Standard C++ Header Files */
#include <iostream>

using std::cerr;
using std::cin;
using std::cout;
using std::endl;

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

static void usage(int bExit, char const *reason, int iInvalidArg, int argc, char **argv);

static void	WINAPI ASSWatcherServiceMain(DWORD dwNumServicesArgs, LPSTR *lpServiceArgVectors);
static void	WINAPI ASSWatcherHandler(DWORD dwControl);

/* /////////////////////////////////////////////////////////////////////////////
 * 
 */

static SERVICE_TABLE_ENTRY	serviceEntries[] = 
{
		{	"ASSWatcher",	ASSWatcherServiceMain	}
	,	{	NULL,			NULL					}
};

/* ////////////////////////////////////////////////////////////////////////// */

int main_(int argc, char *argv[])
{
#if 0
	::Sleep(100000);
#endif /* 0 */

    int                 iRet    =   0;
	int					bVerbose	=	true;
    int                 i;

#if defined(_DEBUG) || \
    defined(__SYNSOFT_DBS_DEBUG)
	cout << "winstl_service_test: " << __STLSOFT_COMPILER_LABEL_STRING << endl;
#endif /* debug */

	for(i = 1; i < argc; ++i)
	{
		const char	*arg	=	argv[i];

		if(arg[0] == '-')
		{
			if(arg[1] == '-')
			{
				/* -- arguments */
			}
			else
			{
				/* - arguments */
				switch(arg[1])
				{
					case	'?':
						usage(1, NULL, -1, argc, argv);
						break;
					case	's':
						bVerbose	=	false;
						break;
					case	'v':
						bVerbose	=	true;
						break;
					default:
						usage(1, "Invalid argument(s) specified", i, argc, argv);
						break;
				}
			}
		}
		else
		{
			/* other arguments */
			usage(1, "Invalid argument(s) specified", i, argc, argv);
		}
	}

	/* . */

#if 1
	winstl::services::manager	mgr(NULL, NULL);
//	SERVICE_STATUS				status;

	mgr.DeleteService("ASS Watcher");

//	winstl::services::service	svc(mgr.OpenService("Alerter"));
	winstl::services::service	svc(mgr.CreateService("ASS Watcher", "ASS Watcher", argv[0]));

//	svc.DeleteService();

//	svc.ControlService(SERVICE_CONTROL_PAUSE);
//	svc.ControlService(SERVICE_CONTROL_CONTINUE);
#endif /* 0 */

	cout << "Starting service control dispatcher\n";

	if(!StartServiceCtrlDispatcher(serviceEntries))
	{
		throw winstl::services::service_exception("failed to start the service control dispatcher", ::GetLastError());
	}

    return iRet;
}

/* ////////////////////////////////////////////////////////////////////////// */

static void usage(int bExit, char const *reason, int iInvalidArg, int argc, char **argv)
{

	cerr << endl;
    cerr << "  winstl_service_test" << endl << endl;
	cerr << endl;

	if(NULL != reason)
	{
		cerr << "  Error: " << reason << endl;
		cerr << endl;
	}

#if 0
    cerr << "  winstl_service_test" << endl << endl;
    cerr << "  USAGE: winstl_service_test [{-w | -p<root-paths> | -h}] [-u] [-d] [{-v | -s}] <search-spec>" << endl;
    cerr << "    where:" << endl << endl;
    cerr << "    -w             - searches from the current working directory. The default" << endl;
    cerr << "    -p<root-paths> - searches from the given root path(s), separated by \';\'," << endl;
    cerr << "                     eg." << endl;
    cerr << "                       -p\"c:\\windows;x:\\bin\"" << endl;
    cerr << "    -r             - deletes readonly files" << endl;
    cerr << "    -h             - searches from the roots of all drives on the system" << endl;
    cerr << "    -d             - displays the path(s) searched" << endl;
    cerr << "    -u             - do not act recursively" << endl;
    cerr << "    -v             - verbose output. Prints time, attributes, size and path. (default)" << endl;
    cerr << "    -s             - succinct output. Prints path only" << endl;
    cerr << "    <search-spec>  - one or more file search specifications, separated by \';\'," << endl;
    cerr << "                     eg." << endl;
    cerr << "                       \"*.exe\"" << endl;
    cerr << "                       \"myfile.ext\"" << endl;
    cerr << "                       \"*.exe;*.dll\"" << endl;
    cerr << "                       \"*.xl?;report.*\"" << endl;
	cerr << endl;
	cerr << "  Contact " << _nameSynesisSoftware << endl;
	cerr << "    at \"" << _wwwSynesisSoftware_SystemTools << "\"," << endl;
	cerr << "    or, via email, at \"" << _emailSynesisSoftware_SystemTools << "\"" << endl;
	cerr << endl;
#endif /* 0 */

    if(bExit)
    {
        exit(EXIT_FAILURE);
    }
}

int main(int argc, char *argv[])
{
	try
	{
		return main_(argc, argv);
	}
	catch(winstl::services::service_exception &x)
	{
		cerr << "Service exception: " << x.what() << "; error = " << x.last_error() << "; " << winstl::error_desc(x.last_error()) << endl;
	}
	catch(std::exception &x)
	{
		cerr << "Unhandled error: " << x.what() << endl;
	}
	catch(...)
	{
		cerr << "Unhandled unknown error" << endl;
	}

	return EXIT_FAILURE;
}

/* /////////////////////////////////////////////////////////////////////////////
 * Service functions
 */

static void	WINAPI ASSWatcherServiceMain(DWORD dwNumServicesArgs, LPSTR *lpServiceArgVectors)
{
	cout << "ASSWatcherServiceMain()" << endl;

	SERVICE_STATUS_HANDLE	hStatus	=	::RegisterServiceCtrlHandler("ASSWatcher", ASSWatcherHandler);

}

static void	WINAPI ASSWatcherHandler(DWORD dwControl)
{
	cout << "ASSWatcherHandler(" << dwControl << ")" << endl;

	switch(dwControl)
	{
		case	SERVICE_CONTROL_PAUSE:
			break;
	}
}

/* ////////////////////////////////////////////////////////////////////////// */
