/* /////////////////////////////////////////////////////////////////////////////
 * File:        xmlstl_test.cpp
 *
 * Purpose:     Implementation file for the xmlstl_test project.
 *
 * Created:     21st October 2004
 * Updated:     11th November 2004
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright (C) 1999-2004, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              This source code is placed into the public domain 2004
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software. 
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage. 
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission. 
 *
 * ////////////////////////////////////////////////////////////////////////// */

/* STLSoft Header Files */
#include <stlsoft.h>
#include <stlsoft_simple_string.h>
#include <stlsoft_string_tokeniser.h>

/* Synesis Header Files */
#include <MLBase.h>
#include <MLRelItf.h>

/* COMSTL Header Files */
#include <comstl.h>
#include <comstl_initialisers.h>
#include <comstl/bstr.hpp>

/* WinSTL Header Files */
#include <winstl.h>
#include <winstl_char_conversions.h>
#include <winstl_error_desc.h>

/* XMLSTL Header Files */
#include <xmlstl.h>

#define USE_MSXML_DOM
//#define USE_XERCES_DOM

#include <xmlstl/msxml/msxml.hpp>
#if defined(USE_MSXML_DOM)
# include <xmlstl/msxml/dom/attribute_sequence.hpp>
# include <xmlstl/msxml/dom/child_node_sequence.hpp>
# include <xmlstl/msxml/dom/node.hpp>
# include <xmlstl/msxml/dom/document.hpp>
# include <xmlstl/msxml/dom/exception.hpp>
# include <xmlstl/msxml/dom/functions.hpp>
# include <xmlstl/msxml/dom/node_sequence_iterator.hpp>
# include <xmlstl/msxml/dom/parse_error.hpp>
# include <xmlstl/msxml/dom/parse_error_info.hpp>
#elif defined(USE_XERCES_DOM)
# include <xmlstl/msxml/dom/attribute_sequence.hpp>
# include <xmlstl/msxml/dom/child_node_sequence.hpp>
# include <xmlstl/msxml/dom/node_sequence_iterator.hpp>
#else
# error Unspecified 
#endif /*  */

/* Standard C Header Files */
#include <stdio.h>
#include <stdlib.h>

/* Standard C++ Header Files */
#include <iostream>
#include <iomanip>

using std::cerr;
using std::cin;
using std::cout;
using std::endl;

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

static void usage(int bExit, char const *reason, int iInvalidArg, int argc, char **argv);


/* /////////////////////////////////////////////////////////////////////////////
 * Version information constants
 */

extern "C"
{
const int  _mccVerHi   =   1;
const int  _mccVerLo   =   0;
const int  _mccVerRev  =   1;
const int  _mccBldNum  =   1;
} // extern "C"

#define	COMPANY_NAME	"Synesis Software"
#define	TOOL_NAME		"XMLSTL Text"
#define	EXE_NAME		"xmlstl_test"

/* ////////////////////////////////////////////////////////////////////////// */

int main_(int argc, char *argv[])
{
#if 0
	::Sleep(100000);
#endif /* 0 */

    int         iRet    =   0;
	int			bVerbose	=	true;
    int         i;
	char const	*xmlFile	=	NULL;

#if defined(_DEBUG) || \
    defined(__SYNSOFT_DBS_DEBUG)
	puts("xmlstl_test: " __STLSOFT_COMPILER_LABEL_STRING);
#endif /* debug */

	for(i = 1; i < argc; ++i)
	{
		const char	*arg	=	argv[i];

		if(arg[0] == '-')
		{
			if(arg[1] == '-')
			{
				/* -- arguments */
				usage(1, "Invalid argument(s) specified", i, argc, argv);
			}
			else
			{
				/* - arguments */
				switch(arg[1])
				{
					case	's':
						bVerbose	=	false;
						break;
					case	'v':
						bVerbose	=	true;
						break;
					default:
						usage(1, "Invalid argument(s) specified", i, argc, argv);
						break;
				}
			}
		}
		else
		{
			if(NULL == xmlFile)
			{
				xmlFile = arg;
			}
			else
			{
				/* other arguments */
				usage(1, "Invalid argument(s) specified", i, argc, argv);
			}
		}
	}

	/* . */
	if(NULL == xmlFile)
	{
		usage(1, "Must specify an XML file to load", -1, argc, argv);
	}
	else
	{
		::comstl::com_initialiser	init;

		try
		{
			xmlstl::msxml::dom::IXMLDOMDocument_ptr		doc_ptr	=	xmlstl::msxml::dom::DOM_load_document(::comstl::ex::bstr(xmlFile));

			xmlstl::msxml::dom::document				doc(doc_ptr);

			xmlstl::msxml::dom::string_t				text	=	doc.text;
		}
		catch(xmlstl::msxml::dom::parse_error &x)
		{
			xmlstl::msxml::dom::parse_error_info	pei(x.pe());

			cout << "DOM Exception: reason: " << pei.reason() << " in " << pei.url() << "(" << pei.line() << ", " << pei.linepos() << ")" << endl;
		}
		catch(xmlstl::msxml::dom::exception &x)
		{
			cerr << "DOM Exception: " << x.what() << ": 0x" << std::hex << x.hr() << " (" << ::winstl::error_desc(x.hr(), stlsoft::string_tokeniser<stlsoft::simple_string, char>("ole32.dll;msxml4.dll;msxml3.dll;msxml2.dll;msxml.dll", ';')) << ")" << endl;
		}
	}

    return iRet;
}

/* ////////////////////////////////////////////////////////////////////////// */

static void usage(int bExit, char const *reason, int iInvalidArg, int argc, char **argv)
{
    fprintf(stderr, "\n");
	fprintf(stderr, "  %s %s Tool, v%d.%d.%d.%04d\n", COMPANY_NAME, TOOL_NAME, _mccVerHi, _mccVerLo, _mccVerRev, _mccBldNum);
    fprintf(stderr, "\n");

	if(NULL != reason)
	{
		fprintf(stderr, "  Error: %s\n", reason);
		fprintf(stderr, "\n");
	}

    fprintf(stderr, "  USAGE 1: %s <command> [<command-args-0> ... <command-args-N>]\n\n", EXE_NAME);
	fprintf(stderr, "    <command>       -   the program to execute\n");
	fprintf(stderr, "    <command-args>  -   all the arguments, if any, of the program\n\n");
    fprintf(stderr, "\n");
    fprintf(stderr, "    Executes the given program and causes all its output, whether from\n");
    fprintf(stderr, "    the standard output or the standard error stream, to be directed\n");
    fprintf(stderr, "    to the standard output stream\n");
    fprintf(stderr, "\n");
    fprintf(stderr, "  USAGE 2: %s -?\n", EXE_NAME);
    fprintf(stderr, "\n");
    fprintf(stderr, "    Displays this help\n");
    fprintf(stderr, "\n");

    if(bExit)
    {
        exit(EXIT_FAILURE);
    }
}

int main(int argc, char *argv[])
{
	try
	{
		return main_(argc, argv);
	}
	catch(std::exception &x)
	{
		cerr << "Unhandled error: " << x.what() << endl;
	}
	catch(...)
	{
		cerr << "Unhandled unknown error" << endl;
	}

	return EXIT_FAILURE;
}

/* ////////////////////////////////////////////////////////////////////////// */
